import { init } from 'echarts-comp/core';
import React, { memo, useEffect, useRef, useMemo, useLayoutEffect, useCallback } from 'react';

const ECharts = memo(function ECharts(props) {
    const resizeDeps = props.resizeDep ?? [];
    const { chartElement, echarts, resize } = useECharts(props);
    useEffect(resize, resizeDeps);
    return chartElement;
});
function createEchartsDom() {
    const dom = document.createElement("div");
    dom.style.overflow = "hidden";
    dom.style.height = "100%";
    return dom;
}
/** When the theme or initOption changes (shallow comparison), reinitialize Echarts */
function useECharts(config = {}) {
    const { fixedSize, loading, onChange, option, init: init$1, style } = config;
    const chartRef = useRef();
    const needInit = useEchartsNeedInit(init$1); // 浅比较
    const oldOption = useOldValue(config.option);
    const dom = useMemo(createEchartsDom, []);
    const echarts = useMemo(() => {
        const oldInstance = chartRef.current;
        let echarts;
        if (oldInstance) {
            oldInstance.dispose();
            echarts = init(dom, init$1?.theme, init$1); //auto
            oldOption && echarts.setOption(oldOption, { lazyUpdate: true });
        }
        else {
            echarts = init(dom, init$1?.theme, { ...init$1, width: 1, height: 1 });
        }
        chartRef.current = echarts;
        onChange?.(echarts, oldInstance);
        return echarts;
    }, [needInit]);
    //update options
    useLayoutEffect(() => {
        if (objectIsEqual(oldOption, option))
            return;
        updateEchartsOption(echarts, option ?? {});
    });
    //update loading
    useLayoutEffect(refreshLoading, [loading]);
    //unmount
    useLayoutEffect(() => {
        return () => chartRef.current.dispose();
    }, []);
    //listen resize
    useEffect(() => {
        if (!fixedSize) {
            const onResize = () => {
                if (timeRef.current !== undefined)
                    return;
                timeRef.current = setTimeout(() => {
                    timeRef.current = undefined;
                    resize();
                }, 40);
            };
            window.addEventListener("resize", onResize);
            return () => {
                //Maybe "Instance xxx has been disposed"
                if (timeRef.current !== undefined)
                    clearTimeout(timeRef.current);
                window.removeEventListener("resize", onResize);
            };
        }
    }, [fixedSize]);
    const timeRef = useRef();
    function refreshLoading() {
        const chart = chartRef.current;
        if (!chart)
            return;
        if (loading)
            chart.showLoading();
        else
            chart.hideLoading();
    }
    const resize = useCallback(() => {
        const echarts = chartRef.current;
        const parent = dom.parentNode;
        if (!parent)
            return;
        echarts.resize({ width: dom.clientWidth, height: dom.clientHeight });
    }, []);
    const onDomReady = useCallback((container) => {
        if (container) {
            container.appendChild(dom);
            resize();
        }
        else {
            //容器被卸载
            dom.parentNode?.removeChild(dom);
        }
    }, []);
    const chartElement = React.createElement("div", { ref: onDomReady, style: { height: "100%", ...style } });
    return { chartElement, echarts, resize, isReady: () => Boolean(dom.parentElement) };
}
function updateEchartsOption(instance, opts) {
    instance.setOption(opts ?? {}, true, true);
}
function useOldValue(value) {
    const ref = useRef();
    const oldValue = ref.current;
    ref.current = value;
    return oldValue;
}
function useEchartsNeedInit(initOpts) {
    const oldOpts = useOldValue(initOpts);
    const changeObj = useRef({});
    if (!objectIsEqual(oldOpts, initOpts)) {
        changeObj.current = {};
    }
    return changeObj.current;
}
function objectIsEqual(obj1, obj2) {
    if (obj1 === obj2)
        return true;
    if (typeof obj1 !== "object" || typeof obj2 !== "object")
        return obj1 === obj2;
    if (obj1 === null)
        return obj1 === obj2;
    const keys1 = Object.keys(obj1);
    const keys2 = Object.keys(obj2);
    if (keys1.length !== keys2.length)
        return false;
    for (let i = 0; i < keys2.length; i++) {
        if (obj1[keys2[i]] !== obj2[keys2[i]])
            return false;
    }
    return true;
}

export { ECharts, useECharts };
